﻿using System;
using System.Linq;
using System.Windows.Forms;

/* Approximation of a function f(x,y) with quadratic input as supporting points  */
/* using the method of the least spuares for multiple input features             */
/*                       www.mosismath.com                                       */

namespace WindowsFormsApplication1
{
    
    public partial class Mainwin : Form
    {
        int samples = 0;
        int dataLength = 0;
        const int features = 5;
        double[,] fOfX;
        double[] x = new double[features];
        Data data = new Data();
   
        public Mainwin()
        {
            int i, j;
            DataGridViewCell cell;

            InitializeComponent();

            data.readData(Application.StartupPath + "\\data.csv");
            gSolution.ColumnCount = features;
            gSolution.RowCount = 1;
            samples = data.values.Count;
            dataLength = data.dataLength;
            dgDataView.RowCount = samples;
            dgDataView.ColumnCount = features;

            // create and fill the matrix f(x)
            fOfX = new double[samples, features];
            double[] tempD;
            for (i = 0; i < samples; i++)
            {
                tempD = data.values.ElementAt(i);
                fOfX[i, 0] = tempD[0];
                fOfX[i, 1] = tempD[0] * tempD[0];

                fOfX[i, 2] = tempD[1];
                fOfX[i, 3] = tempD[1] * tempD[1];
                fOfX[i, 4] = 1.0;
            }

            dgDataView.Columns[0].HeaderText = "x";
            dgDataView.Columns[1].HeaderText = "y";
            dgDataView.Columns[2].HeaderText = "z";
            dgDataView.Columns[3].HeaderText = "f(x,y)";
            dgDataView.Columns[4].HeaderText = "dev [%]";
            for (i = 0; i < samples; i++)
            {
                dgDataView.Rows[i].HeaderCell.Value = (i + 1).ToString();
                cell = dgDataView[0, i];
                cell.Value = Math.Round(data.values.ElementAt(i)[0], 4).ToString();
                cell = dgDataView[1, i];
                cell.Value = Math.Round(data.values.ElementAt(i)[1], 4).ToString();
                cell = dgDataView[2, i];
                cell.Value = Math.Round(data.values.ElementAt(i)[2], 4).ToString();
            }
        }

        private void bt_Calc_Click(object sender, EventArgs e)
        {
            int i, j, k;
            
            double xRes;
            double y;
            double z;

            //TMatrix m = new TMatrix(features);
            double[,] m = new double[features, features];
            double[] d = new double[features];
            TGivens givens = new TGivens(features, m, d);
            DataGridViewCell cell;

            // fill the matrix
            for (i = 0; i < features; i++)
            {
                for (j = 0; j < features; j++)
                {
                    for (k = 0; k < samples; k++)
                        m[j, i] = m[j, i] + fOfX[k, i] * fOfX[k, j];
                }
                for (k = 0; k < samples; k++)
                    d[i] = d[i] + data.values.ElementAt(k)[2] * fOfX[k, i];
            }

            // solve the matrix equation by Givens transformations
            givens.RunGivens();
            x = givens.SolveRMatrix();
           
            // display the results
            for (i = 0; i < features; i++)
            {
                gSolution.Columns[i].HeaderText = "c" + (i + 1).ToString();
                cell = gSolution[i, 0];
                cell.Value = Math.Round(x[i], 4).ToString();
            }
          
            if ((x[1] != 0) && (x[3] != 0))
            {
                xRes = -x[0] / x[1] / 2;
                y = -x[2] / x[3] / 2;
                z = xRes * x[0] + xRes * xRes * x[1] + y * x[2] + y * y * x[3] + x[4];
                tbX0.Text = Math.Round(xRes, 4).ToString();
                tbY0.Text = Math.Round(y, 4).ToString();
                tbZ0.Text = Math.Round(z, 4).ToString();

                for (i = 0; i < samples; i++)
                {
                    xRes = data.values.ElementAt(i)[0];
                    y = data.values.ElementAt(i)[1];
                    z = xRes * x[0] + xRes * xRes * x[1] + y * x[2] + y * y * x[3] + x[4];
                    cell = dgDataView[3, i];
                    cell.Value = Math.Round(z, 4).ToString();
                    cell = dgDataView[4, i];
                    if (z != 0)
                        cell.Value = Math.Round((z - data.values.ElementAt(i)[2]) / z * 100, 4).ToString();
                }
            }
            else
                MessageBox.Show("No minimum found for Z");
        }
    }
}
