﻿using System;


namespace WindowsFormsApplication1
{
   
    public class TGivens
    {
        double[,] m;
        double[] d;
        double[,] r;
        int maxOrder;

        public TGivens(int order, double[,] matrix, double[] solution)
        {
            maxOrder = order;
            m = matrix; // equation Matrix
            d = solution;
            r = new double[maxOrder, maxOrder]; // rotation Matrix
        }

        public void CalcRMatrix(int p, int q)
        {
            int i, j;
            double radius;
            radius = Math.Sqrt(m[p, p] * m[p, p] + m[q, p] * m[q, p]);
            for (i = 0; i < maxOrder; i++)
            {
                for (j = 0; j < maxOrder; j++)
                {
                    r[i, j] = 0.0;
                }
                r[i, i] = 1.0;
            }
            if (radius != 0.0)
            {
                if (m[p, p] == 0.0)
                {
                    r[p, p] = 0.0;
                    r[p, q] = 1.0;
                }
                else
                {
                    //  r.c[p, p] = Math.Abs(m.c[p, p]) / radius;
                    //  r.c[p, q] = Math.Sign(m.c[p, p]) * m.c[q, p] / radius;
                    r[p, p] = m[p, p] / radius;
                    r[p, q] = m[q, p] / radius;
                }
                r[q, q] = r[p, p];
                r[q, p] = -r[p, q];
            }
        }


        public void RunGivens()
        {
            int i, k;
            for (i = 0; i < maxOrder; i++)
            {
                for (k = i + 1; k < maxOrder; k++)
                {
                    if (m[k, i] != 0.0)
                    {
                        CalcRMatrix(i, k);
                        MultMatrix(r, i, k);
                    }
                }
            }
        }

        public double[] SolveRMatrix()
        {
            int k, l;
            double[] x = new double[maxOrder];
            for (k = maxOrder - 1; k >= 0; k--)
            {
                for (l = maxOrder - 1; l > k; l--)
                {
                    d[k] = d[k] - x[l] * m[k, l];
                }
                if (m[k, k] != 0)
                    x[k] = d[k] / m[k, k];
                else
                    x[k] = 0;
            }
            return x;
        }

        public void MultMatrix(double[,] matrix2, int p, int q)
        {
            int i, j, k;
            double[,] al = new double[maxOrder, maxOrder];
            double[] yl = new double[maxOrder];
            for (i = 0; i < maxOrder; i++)
            {
                if ((i == p) || (i == q)) // only calculate row p and q
                {
                    yl[i] = 0.0;
                    for (j = 0; j < maxOrder; j++)
                    {
                        yl[i] = yl[i] + (d[j] * matrix2[i, j]);
                        al[i, j] = 0.0;
                        for (k = 0; k < maxOrder; k++)
                        {
                            al[i, j] = al[i, j] + (matrix2[i, k] * m[k, j]);
                        }
                    }
                }
                else
                {
                    yl[i] = d[i];
                    for (j = 0; j < maxOrder; j++)
                    {
                        al[i, j] = m[i, j];
                    }
                }
            }
            for (i = 0; i < maxOrder; i++)
            {
                d[i] = yl[i];
                for (j = 0; j < maxOrder; j++)
                {
                    m[i, j] = al[i, j];
                }
            }
        }
    }
}
